/**
* PETALS - PETALS Services Platform.
* Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
* -------------------------------------------------------------------------
* $Id$
* -------------------------------------------------------------------------
*/

package org.ow2.petals.bpel.engine;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;

import org.ow2.petals.bpel.engine.env.JBIReceiverImpl;
import org.ow2.petals.bpel.engine.env.JBISenderImpl;
import org.ow2.petals.bpel.engine.message.JBIExternalMessageImpl;
import org.ow2.petals.bpel.engine.message.JbiAndBpelMessageConverterImpl;
import org.ow2.petals.bpel.engine.su.CoreServiceUnitManager;
import org.ow2.petals.component.framework.jbidescriptor.generated.Jbi;
import org.ow2.petals.component.framework.se.AbstractServiceEngine;
import org.ow2.petals.component.framework.su.AbstractServiceUnitManager;
import org.w3c.dom.Element;

import com.ebmwebsourcing.easybpel.model.bpel.BPELFactory;
import com.ebmwebsourcing.easybpel.model.bpel.api.BPELException;
import com.ebmwebsourcing.easybpel.model.bpel.impl.message.BPELInternalMessageImpl;
import com.ebmwebsourcing.easyviper.core.api.Core;
import com.ebmwebsourcing.easyviper.core.api.engine.configuration.ConfigurationEngine;
import com.ebmwebsourcing.easyviper.core.api.soa.message.MessageConverter;
import com.ebmwebsourcing.easyviper.core.impl.engine.configuration.ConfigurationEngineImpl;

/**
* The Core Service Engine component class.
* @author Nicolas SALATGE - eBM WebSourcing
*
*/
public class BPELEngine extends AbstractServiceEngine {

   /**
    * the environment Factory
    */
   private Core core;

   /**
    * The default constructor
    *
    */
   public BPELEngine() {
       super();
   }

   /**
    * initialize the component
    *
    * @throws JBIException
    *             impossible to initialize the component
    */
   @Override
   protected final void doInit() throws JBIException {
       try {
           super.doInit();

           this.getLogger().log(Level.INFO, "Core Intialization...");

                      // create internal message type
           final BPELInternalMessageImpl internalMessage = new BPELInternalMessageImpl();

           // create external message type
           final JBIExternalMessageImpl externalMessage = new JBIExternalMessageImpl();

           // create message converter
           final MessageConverter<JBIExternalMessageImpl, BPELInternalMessageImpl> converter = new JbiAndBpelMessageConverterImpl(externalMessage.getClass(), internalMessage.getClass());
           
           // get properties
           ConfigurationEngine conf = new ConfigurationEngineImpl();
           if(this.getComponentExtensions().get("explorer") != null) {
               conf.setExplorer(Boolean.valueOf(this.getComponentExtensions().get("explorer")));
           }
           if(this.getComponentExtensions().get("autoFlushMessage-frequency") != null) {
               conf.setAutoFlushMessageFreqeuency(Integer.parseInt(this.getComponentExtensions().get("autoFlushMessage-frequency")));
               this.getLogger().log(Level.INFO, "auto-flushMessage activated at frequency: " + conf.getAutoFlushMessageFrequency());
           }
           if(this.getComponentExtensions().get("autoTrashProcess-frequency") != null) {
               conf.setAutoTrashProcessFreqeuency(Integer.parseInt(this.getComponentExtensions().get("autoTrashProcess-frequency")));
               this.getLogger().log(Level.INFO, "auto-trashProcess activated at frequency: " + conf.getAutoTrashProcessFrequency());
           }
           boolean activateInitialLogger = false;
           if(this.getComponentExtensions().get("internal-logger") != null) {
               activateInitialLogger = Boolean.valueOf(this.getComponentExtensions().get("internal-logger"));
           }
           Logger internalLogger = null;
           if(activateInitialLogger) {
               internalLogger = this.getLogger();
           }
         
                      // Create the core
           this.core = BPELFactory.newInstance().newBPELEngine(conf, converter, 10, JBIReceiverImpl.class, 10, JBISenderImpl.class, internalLogger);
       } catch (final BPELException e) {
           e.printStackTrace();
           throw new JBIException(e);
       }
   }

   public Core getCore() {
       return this.core;
   }
      @Override
   protected AbstractServiceUnitManager createServiceUnitManager() {
       return new CoreServiceUnitManager(this);
   }

   /**
    * Get a parameter.
    * TODO remove this method, use ConfigurationExtensions instead from the CDK API.
    * @param name
    * @param value
    * @param jbi
    */
   public String getParam(String name, Jbi jbi) {
       for (Element element : jbi.getComponent().getAny()) {
           if (element.getLocalName().equals(name)) {
               return element.getTextContent();
           }
       }
              return null;
   }
         }

