/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.bpel.engine.su;

import java.net.URI;
import java.util.List;

import org.ow2.petals.bpel.engine.BPELEngine;
import org.ow2.petals.bpel.engine.util.Util;
import org.ow2.petals.component.framework.AbstractComponent;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.jbidescriptor.generated.Jbi;
import org.ow2.petals.component.framework.jbidescriptor.generated.Provides;
import org.ow2.petals.component.framework.su.AbstractServiceUnitManager;
import org.ow2.petals.component.framework.su.ServiceUnitDataHandler;

import com.ebmwebsourcing.easyviper.core.api.model.Model;
import com.ebmwebsourcing.easyviper.core.api.model.registry.definition.ProcessContextDefinition;
import com.ebmwebsourcing.easyviper.core.impl.model.registry.ProcessContextDefinitionImpl;

/**
 * This class implements the orchestra su manager
 *
 * @author Nicolas SALATGE - eBMWebSourcing
 * @author Adrien LOUIS - eBMWebSourcing
 *
 */
public final class CoreServiceUnitManager extends AbstractServiceUnitManager{// implements ServiceUnitListener {

    /**
     * BPEL property
     */
    private static final String BPEL_PROPERTY = "bpel";

    private static final String POOLSIZE_PROPERTY = "poolsize";

    private static final String FORCE_UNINSTALL_PROPERTY = "force-uninstall";

    /**
     * Default constructor.
     * @param component
     */
    public CoreServiceUnitManager(AbstractComponent component) {
        super(component);
    }


    @Override
    protected void doDeploy(final String serviceUnitName, final String suRootPath,
            final Jbi jbiDescriptor) throws PEtALSCDKException {
        try {
            this.logger
            .info("Perform BPEL-Processes deployment on Core server");

            // Deploy the BPEL process
            final List<Provides> providesList = jbiDescriptor.getServices()
            .getProvides();
            for (final Provides provides : providesList) {
                final ConfigurationExtensions props = new ConfigurationExtensions(
                        provides.getAny());

                // get parameters from extension
                final String bpelProcessName = props
                .get(CoreServiceUnitManager.BPEL_PROPERTY);


                // get BPEL parameter: number of instances to deploy
                int poolSize = 0;
                if(props.get(CoreServiceUnitManager.POOLSIZE_PROPERTY) != null) {
                      poolSize = Integer.valueOf(props.get(CoreServiceUnitManager.POOLSIZE_PROPERTY));//Integer.valueOf(this.getComponentExtensions().get("poolsize"));
                }
                ProcessContextDefinition context=  new ProcessContextDefinitionImpl();
                context.setPoolSize(poolSize);


                // Store the BPEL process       
                URI uri = Util.getResourceAsURL(suRootPath, bpelProcessName).toURI();
                Model model = ((BPELEngine)this.component).getCore().getModel();
                model.getRegistry().storeProcessDefinition(uri, context);//, );
                this.logger
                .info("The BPEL Process " + bpelProcessName + " has been succesfully deployed on Core server and is available on the endpoint "
                        + provides.getServiceName()
                        + provides.getEndpointName());
            }
            this.logger
            .info("BPEL-Processes deployment on Core server has been performed");
        } catch (final Exception e) {
            throw new PEtALSCDKException(e);
        }
    }

    @Override
    protected void doShutdown(final String serviceUnitName) throws PEtALSCDKException {
        // do nothing
    }

    @Override
    protected void doStart(final String serviceUnitName) throws PEtALSCDKException {

        // do nothing
    }

    @Override
    protected void doStop(final String serviceUnitName) throws PEtALSCDKException {
        // do nothing
    }


    @Override
    protected void doUndeploy(final String serviceUnitName) throws PEtALSCDKException {
        try {
            this.logger
            .info("Perform BPEL-Processes undeployment on Core server");
            ServiceUnitDataHandler suDataHandler = this.getServiceUnitDataHandlers().get(serviceUnitName);
            for (final Provides provides : suDataHandler.getDescriptor().getServices().getProvides()) {
                final ConfigurationExtensions props = suDataHandler.getConfigurationExtensions(provides);

                // get parameters from extension
                final String bpelProcessName = props
                .get(CoreServiceUnitManager.BPEL_PROPERTY);

                // un-store the BPEL process
                Boolean forceUninstall = true;
                if(props.get(CoreServiceUnitManager.FORCE_UNINSTALL_PROPERTY) != null) {
                    forceUninstall = Boolean.valueOf(props.get(CoreServiceUnitManager.FORCE_UNINSTALL_PROPERTY));
                } 

                //TODO roland : the SU install root is always a file location, not a URL.
                ((BPELEngine)this.component).getCore().getModel().getRegistry().unStoreProcessDefinition(Util.getResourceAsURL(suDataHandler.getInstallRoot(), bpelProcessName).toURI(), forceUninstall);

                this.logger.info("The BPEL Process "
                        + bpelProcessName
                        + " has been succesfully undeployed on Core server");
            }
            this.logger
            .info("BPEL-Processes undeployment on Core server has been performed");
        } catch (final Exception e) {
            throw new PEtALSCDKException(e);
        }
    }
}

