/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.bpel.engine.util;

import java.net.URI;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.activation.DataHandler;
import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessagingException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.security.auth.Subject;
import javax.xml.namespace.QName;
import javax.xml.transform.dom.DOMSource;

import org.jdom.input.DOMBuilder;
import org.ow2.petals.bpel.engine.listener.JBIListener;
import org.ow2.petals.bpel.engine.message.JBIContextImpl;
import org.ow2.petals.bpel.engine.message.JBIExternalMessageImpl;
import org.ow2.petals.component.framework.api.Message;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.util.UtilFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.ebmwebsourcing.easybpel.model.bpel.api.BPELException;
import com.ebmwebsourcing.easybpel.model.bpel.api.message.BPELInternalMessage;
import com.ebmwebsourcing.easybpel.model.bpel.impl.message.BPELInternalMessageImpl;
import com.ebmwebsourcing.easyviper.core.api.CoreException;
import com.ebmwebsourcing.easyviper.core.api.env.ExternalContext;
import com.ebmwebsourcing.easyviper.core.api.soa.Endpoint;

/**
 * @author nicolas salatge - ebmm websourcing
 * @author alouis
 *
 */
public class Sender {

    private final Logger logger;


    private final ComponentContext context;

    /**
     * Send style
     *
     */
    public static enum SendStyle {
        ASYNCHRONOUS_SEND, SYNCHRONOUS_SEND, SYNCHRONOUS_SEND_WITH_TIMEOUT
    };

    /**
     * Default constructor
     *
     */
    public Sender(final ComponentContext context,
            final DeliveryChannel channel, final Logger logger) {

        this.logger = logger;

        this.context = context;
    }

    /**
     * FIXME some orchestra Error and Fault exception are thrown FIXME use some
     * CDK utils to send Exchanges Call a jbi service
     *
     * @param partner
     *            the partner to invoke. May be null
     * @param msgIn
     *            the exchange
     * @param syncTime
     *            timeout for syncSend
     * @return the exchange response
     * @throws CoreException
     * @throws MessagingException
     *             problem while accessing MsgExch information or while sending
     *             the MsgExc
     * @throws OrchestraEngineException no endpoint found to send the message
     */
    public final JBIExternalMessageImpl send(final JBIExternalMessageImpl internalMsg,
            final Long syncTime, final Endpoint providerEndpointInfo, final ExternalContext context, final SendStyle sendStyle) throws CoreException {
        JBIExternalMessageImpl messageReturned = null;
        try {
            if(providerEndpointInfo == null) {
                throw new BPELException("The provider endpoint information cannot be null");
            }

            // retrieve the jbi context
            final JBIContextImpl jbiContext = (JBIContextImpl)context;
            JBIListener listener = jbiContext.getListener();

            this.logger.finest("create JBI exchange");
            final Exchange msgIn = this.createMsg(providerEndpointInfo.getInterfaceName(), providerEndpointInfo.getInvokedOperation(), sendStyle, internalMsg.getContent().getOwnerDocument(), jbiContext.getInitialExchange().getInMessageAttachments(), jbiContext.getInitialExchange().getInMessagePropertyNames(), jbiContext.getInitialExchange().getInMessage().getSecuritySubject(), jbiContext.getInitialExchange(), listener);

            this.logger.finest("Send a JBI message");
            String endpointName = null;
            QName service = null;

            //to keep some message stuff needed after its send
            Fault keptFault = null;
            java.lang.Exception keptException = null;
            Element keptElement = null;
            URI keptPattern = null;

            msgIn.setInterfaceName(providerEndpointInfo.getInterfaceName());

            this.logger.finest("message mep : " + msgIn.getPattern());

            endpointName = providerEndpointInfo.getEndpointName();
            this.logger.fine("invoked endpoint name: " + endpointName);
            service = providerEndpointInfo.getServiceName();
            if (service != null) {
                this.logger.fine("Set the service to invoke: " + service);
                msgIn.setService(service);
                if (endpointName != null) {
                    final ServiceEndpoint serviceEndpoint = this.resolveEndPoint(service,
                            endpointName);

                    this.logger.fine("Set the endpoint to invoke: "
                            + serviceEndpoint.getEndpointName());
                    msgIn.setEndpoint(serviceEndpoint);
                }
            } //Retrieve service + endpoint in the address
            else{
                String messAddre =  providerEndpointInfo.getAddress();               
                if(messAddre.contains("::")){
                    String[] tabServiceEndpoint = messAddre.split("::");
                    if(tabServiceEndpoint.length>1){
                        service = QName.valueOf(tabServiceEndpoint[0]);
                        endpointName = tabServiceEndpoint[1];
                        msgIn.setService(service);
                        final ServiceEndpoint serviceEndpoint = this.resolveEndPoint(service,
                                endpointName);
                        msgIn.setEndpoint(serviceEndpoint);
                    }
                }else{
                    this.logger.severe(messAddre + " is not a valid JBI address. It MUST be of the following form: {namespace}ServiceName::EndpointName");
                    if(messAddre.startsWith("http:")){
                        this.logger.severe("You entered a http address !!! Petals BPEL engine is in a bus and can only access JBI addresses");
                    }
                }

            }


            this.logger.finest("Satus: " + msgIn.getStatus());
            this.logger.finest("Consumer Role? " + msgIn.isConsumerRole());

            if ((sendStyle == SendStyle.ASYNCHRONOUS_SEND)
                    && (msgIn.isInOnlyPattern())) {
                listener.send(msgIn);
            } else if (sendStyle == SendStyle.SYNCHRONOUS_SEND) {
                if (syncTime != null) {
                    listener.sendSync(msgIn);
                } else {
                    listener.sendSync(msgIn);
                }

                if(msgIn.isOutMessage()){
                    Document docOutMessage = null;
                    if(msgIn.getOutMessage().getContent() instanceof DOMSource) {
                        docOutMessage = (Document) ((DOMSource)msgIn.getOutMessage().getContent()).getNode();
                    } else {
                        docOutMessage = msgIn.getOutMessageContentAsDocument(true);
                    }
                    if(docOutMessage !=null){
                        keptElement = docOutMessage.getDocumentElement();
                    }
                }



                if(msgIn.getFault()!=null){
                    keptFault = msgIn.getFault();
                }
                if(msgIn.getError()!=null){
                    keptException = msgIn.getError();
                }

                if(msgIn.getPattern()!=null){
                    keptPattern = msgIn.getPattern();
                }

                if (!msgIn.getPattern().equals(Message.MEPConstants.IN_ONLY_PATTERN)) {
                    if(msgIn.getOutMessage()!=null){

                        this.logger.finest("Set status done..."+msgIn.getEndpointName());
                        msgIn.setDoneStatus();
                        this.logger.finest("Send status done");
                    }else{
                        this.logger.finest("Set status error..."+msgIn.getEndpointName());
                        msgIn.setErrorStatus();
                    }





                    listener.send(msgIn);

                }

                if ((keptFault != null) 
                        || (keptException != null)) {
                    if (keptException != null) {
                        throw new CoreException(keptException);
                    }

                    if (keptFault != null) {
                        Document faultDoc = UtilFactory.getSourceUtil().createDocument(keptFault.getContent());

                        BPELInternalMessage faultMsg = new BPELInternalMessageImpl();
                        // convert dom to jdom
                        DOMBuilder builder = new DOMBuilder();
                        org.jdom.Document faultJDoc = builder.build(faultDoc);

                        faultMsg.setContent(faultJDoc.getRootElement());
                        faultMsg.setEndpoint(endpointName);
                        faultMsg.setOperationName(providerEndpointInfo.getInvokedOperation());
                        faultMsg.setService(service);
                        CoreException userEx = new CoreException();
                        userEx.setFault(faultMsg);
                        throw userEx;
                    }
                    throw new CoreException("Error from the provider " + service
                            + endpointName);
                }

                this.logger.log(Level.FINE, "Receive response from: " + service
                        + endpointName);
                if (sendStyle == SendStyle.SYNCHRONOUS_SEND) {
                    messageReturned = new JBIExternalMessageImpl();
                    //  this.logger.finest("messageReturned = " + msgIn);
                    Element responseElmt = null;

                    // TODO: FIX BUG =>
                    // replace all by
                    if(keptElement!=null) {
                        responseElmt = keptElement;
                    }



                    if(responseElmt != null) {
                        messageReturned.setContent(responseElmt);
                    }
                    if (this.logger.isLoggable(Level.FINEST)){
                        this.logger.finest("jbi message returned: " + messageReturned);                    
                    }
                }



            } else {
                // FIXME should not happen : the exchange mep has been created in
                // correlation with the senStyle parameter in a previous method
                throw new CoreException(
                        "this kind of message ("
                        + sendStyle
                        + ") is unknown or is incompatible with the mep of message: "
                        + keptPattern);
            }

            this.logger.finest("JBI message sent");
        } catch (final MessagingException e) {
            this.logger.severe(e.getMessage());
            throw new BPELException(e);
        } catch (PEtALSCDKException e) {
            this.logger.severe(e.getMessage());
            throw new BPELException(e);
        }
        return messageReturned;
    }

    /**
     * Create a JBI message exchange
     *
     * @param itf
     *            the QName
     * @param operation
     *            the operation name
     * @param content
     *            the content of the operation
     * @param attachmentFiles
     *            the list of attachments
     * @param listener
     *            the listener
     * @param properties
     *            the properties
     * @param subject
     *            the orchestra subject
     * @return the exchange. May be null
     * @throws MessagingException
     * @throws PEtALSCDKException
     */
    private Exchange createMsg(final QName itf, final String operation, final SendStyle sendStyle,
            final Document content, final Set<DataHandler> attachments,
            final Set<?> properties, final Subject subject, final Exchange initialExchange, JBIListener listener)
    throws BPELException {
        Exchange exchange = null;
        try {
            Message.MEPConstants mep = Message.MEPConstants.IN_ONLY_PATTERN;

            if (sendStyle == SendStyle.SYNCHRONOUS_SEND) {
                mep = Message.MEPConstants.IN_OUT_PATTERN;
            }
            exchange = this.createExchange(mep, listener);
            exchange.setInterfaceName(itf);

            if (exchange != null) {
                if(itf == null) {
                    throw new BPELException("Interface cannot be null");
                }
                exchange.setOperation(new QName(itf.getNamespaceURI(), operation));
                // setting content
                exchange.setInMessageContent(content);

                if (subject != null) {
                    exchange.getInMessage().setSecuritySubject(subject);

                }

                // Copy properties
                logger.finest("set properties in message: " + properties);
                for(String propertyName: (Set<String>)properties) {
                    logger.finest("set property name: " + propertyName + " with value: " + initialExchange.getInMessageProperty(propertyName));
                    exchange.getInMessage().setProperty(propertyName, initialExchange.getInMessageProperty(propertyName));
                }

                // TODO: Copy attachment
            }
        } catch (final MessagingException e) {
            throw new BPELException(e);
        }
        return exchange;
    }


    /**
     * FIXME use some CDK util methods to create the exchange Create an empty
     * Exchange
     *
     * @param mep
     *            exchange pattern
     * @return
     * @throws MessagingException
     *             JBI factory can not create the MsgExchange
     */
    public Exchange createExchange(final Message.MEPConstants mep, JBIListener listener) throws MessagingException {
        if (mep == null) {
            throw new IllegalArgumentException(
            "'mep' parameter must be non null");
        }
        Exchange exchange = null;
        try {
            if(mep.equals(Message.MEPConstants.IN_ONLY_PATTERN)) {
                exchange = listener.createExchange(Message.MEPConstants.IN_ONLY_PATTERN);
            } else  if(mep.equals(Message.MEPConstants.IN_OUT_PATTERN)) {
                exchange = listener.createExchange(Message.MEPConstants.IN_OUT_PATTERN);
            } else  if(mep.equals(Message.MEPConstants.ROBUST_IN_ONLY_PATTERN)) {
                exchange = listener.createExchange(Message.MEPConstants.ROBUST_IN_ONLY_PATTERN);
            } else  if(mep.equals(Message.MEPConstants.IN_OPTIONAL_OUT_PATTERN)) {
                exchange = listener.createExchange(Message.MEPConstants.IN_OPTIONAL_OUT_PATTERN);
            }
        } catch (PEtALSCDKException e) {
            throw new MessagingException(e);
        }
        return exchange;
    }

    /**
     * Find the endpoint in the JBI env
     *
     * @param service
     * @param endpointName
     * @return
     * @throws OrchestraEngineException
     *             endpoint can not be found in the JBI context
     */
    public final ServiceEndpoint resolveEndPoint(final QName service,
            final String endpointName) throws BPELException {

        this.logger.log(Level.FINEST, "find the endPoint in the JBI environment");

        this.logger.log(Level.FINEST, "service name: " + service);
        this.logger.log(Level.FINEST, "endpoint name: " + endpointName);

        // Search by endpoint
        final ServiceEndpoint endpoint = this.context.getEndpoint(service, endpointName);

        if (endpoint == null) {
            throw new BPELException("endpoint (" + service + "-"
                    + endpointName + ") not found in the JBI environment");
        }

        this.logger.log(Level.FINEST, "endpoint service found: " + endpoint);
        return endpoint;
    }

}

